# Generated by roxygen2: do not edit by hand

export(loadHlaData)
export(loadKirData)
exportMethods(columns)
exportMethods(getAlleles)
exportMethods(getClosestComplete)
exportMethods(getLoci)
exportMethods(getReference)
exportMethods(getStructure)
exportMethods(keys)
exportMethods(keytypes)
exportMethods(select)
import(AnnotationDbi)
import(AnnotationHub)
import(DBI)
import(RSQLite)
import(assertthat)
import(methods)
importFrom(Biostrings,DNAStringSet)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(stats,setNames)
