% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ipdDbClass.R
\docType{methods}
\name{getStructure,IpdDb-method}
\alias{getStructure,IpdDb-method}
\alias{getStructure}
\title{Get gene structures}
\usage{
getStructure(x, allele)
}
\arguments{
\item{x}{The database connection; an \code{\link{IpdDb}} object.}

\item{allele}{The alleles of interest as a character vector.}
}
\value{
A \code{\link[GenomicRanges:GRanges-class]{GenomicRanges:GRanges}} 
object with all gene
  structures.
}
\description{
Get the gene structures for alleles.
}
\examples{
## Load the database 
hla <- loadHlaData()
## Get the loci
loci <- getLoci(hla)
## Get alleles of a locus
alleles <- getAlleles(hla, loci[1])
allelesOfInterest <- alleles[1:10]
## Get the structures
seqs <- getStructure(hla, allelesOfInterest)

}
